<?php
// +----------------------------------------------------------------------
// | 后台基础控制器
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

namespace app\common\controller;

use app\common\controller\Emptys;
use think\facade\Request;
use think\facade\Session;

class _Admin extends Emptys
{
    // 交由子类去触发构造函数
    public function __construct(){
        parent::__construct();
        $this->auth();
    }

    // 代码管理架构
    protected function _autoload($_this) {
        $DS = DIRECTORY_SEPARATOR;
        // 获得控制器名称
        $controller = Request::instance()->controller();
        // 获得操作名称
        $action     = Request::instance()->action();
        // 操作方法文件的绝对地址 
        $url = APP_PATH.'admin'.$DS.'controller'.$DS.$controller.$DS.$action.'.php';
        // 操作方法不存在文件不存在调用基础类404重定向
        if (!file_exists($url)) {
            return $this->__empty();
        }
        // 构造出命名空间
        $url  = "\app\admin\controller\\$controller\\$action";
        // 否则加载该文件
        $obj = new $url();
        $numargs = Request::instance()->param();
        // 统一加载run方法
        echo $obj->run($numargs, $_this);
    }

    /**
     * 不需要检验的路由
    */
    protected $_auth_menu = [
        'index/ziliao',
        'index/pwd',
        'index/heartbeat',
        'index/login',
        'index/notice',
        'index/notice_details',
    ];
    /**
     * 网站配置
    */
    protected $config = [];
    /**
     * 登陆信息
    */
    protected $admin = [];
    /**
     * 权限菜单
    */
    protected $auth_menu = [];

    // 权限验证 
    protected function auth() {
        // 启动模板布局
        $this->view->engine->layout(true);
        // 读取网站配置
        $this->config = \think\Db::name('config')->where('id', 1)->find();
        $this->assign('config', $this->config);

        // 获得控制器名称
        $controller = strtolower(Request::instance()->controller());
        // 获得操作名称
        $action     = strtolower(Request::instance()->action());
        $this->assign('controller', $controller);
        $this->assign('action', $action);
        // 路由地址
        $route = $controller.'/'.$action;
        // 公共方法也不需要校验
        if ($controller == 'common' || $controller == 'login') {
            return true;
        }
        // 登陆信息
        $admin = Session::get('admin');
        // 没登陆先登陆
        if (!$admin) {
            if (Request::isAjax()) {
                echo $this->_json('999', '您已掉线，请重新登录！');
            } else {
                $this->redirect('login/index');
            }
            exit;
        }
        $admin = \think\Db::name('admin')->alias('A')
                 ->join('role B', 'A.role_id=B.id', 'LEFT')
                 ->join('branch C', 'A.branch_id=C.id', 'LEFT')
                 ->where('A.id', $admin['id'])
                 ->field('A.*, B.role_title, C.branch_title')
                 ->find();
        $this->admin = $admin;
        $this->assign('admin', $admin);
        // 心跳掉线
        if (($controller.'/'.$action) != 'index/login' && !empty($admin['heartbeat_time'])) {
            $s = time()-$admin['heartbeat_time'];
            $end = 60*30; // 分钟
            // $end = 20;
            if ($s >= $end) {
                if (Request::isAjax()) {
                    echo $this->_json('998', '登录超时，请输入密码验证身份！');
                } else {
                    $this->redirect('index/login');
                }
                exit;
            }
        }
        // 不需要检验权限
        if (in_array($route, $this->_auth_menu)) {
            return true;
        }
        // 这里就要开始校验权限了
        $this->auth_menu();
        $vif = $this->vif_menu($controller, $action);
        if ($vif === false) {
            if (Request::isAjax()) {
                echo $this->_json('500', '暂无该访问权限！');
            } else {
                $this->error('暂无该访问权限！');
            }
            exit;
        }
        // 将实例传递到模板
        $this->assign('auth', $this);
    }

    // 获取权限列表
    private function auth_menu() {
        if ($this->admin['role_id'] == 1) {
            $this->auth_menu = \think\Db::name('menu')->where('status', 1)->order('pid ASC, sort DESC, id ASC')->select();
        } else {
            $in = '0';
            $list = \think\Db::name('auth')->alias('A')->where('A.role_id', $this->admin['role_id'])->select();
            foreach ($list as $v) {
                $in .= ','.$v['menu_id'];
            }
            $in2 = \think\Db::name('admin_menu')->where('admin_id', $this->admin['id'])->value('menu_list');
            if ($in2) {
                $in .= ','.$in2;
            }

            $this->auth_menu = \think\Db::name('menu')->whereIn('id', $in)->order('pid ASC, sort DESC, id ASC')->select();
        }
    }

    // 校验权限
    public function vif_menu($controller, $action) {
        if ($controller == 'index' && $action == 'index') return true;
        foreach ($this->auth_menu as $v) {
            if ($v['controller'] == $controller && $v['action'] == $action) return true;
        }
        return false;
    }

    // 用于后台输出JSON
    protected function _json($code, $msg, $list=[]) {
        // 写入日志
        if ($this->admin && $msg != '心跳提交成功！' && $msg != '获取成功！') {
            \think\Db::name('admin_action_log')->insert([
                'admin_id' => $this->admin['id'],
                'msg' => $msg,
                'ip' => \think\facade\Request::ip(),
                'create_time' => time(),
            ]);
        }
        return json_encode([
                'code' => "$code",
                'msg' => $msg,
                'data' => $list
            ], JSON_UNESCAPED_UNICODE);
    }

    // 用于后台判断不能为空，并且可以为0的判断
    protected static function isset_empty($param, $key) {
        if (!isset($param[$key])) return false;
        if (trim($param[$key]) == '') return false;
        return true;
    }

    // 用于后台递归生成权限菜单
    protected function menu_make($cate='', $name = 'nodes', $pid = 0) {
        $arr = [];
        foreach ($cate as $v) {
            if ($v['pid'] == $pid) {
                $v[$name] = $this->menu_make($cate, $name, $v['id']);
                $arr[] = $v;
            }
        }
        return $arr;
    }

    /**
     * 输出Json到Table
     * @todo 无
     * @author 小黄牛
     * @version v1.1.5 + 2020.07.14
     * @deprecated 暂不启用
     * @global 无
     * @param array $rows
     * @param int $total
     * @return void
    */
    protected function returnTable($rows, $total) {
        return json_encode([
            'rows' => $rows,
            'total' => $total,
        ], JSON_UNESCAPED_UNICODE);
    }

    /**
     * 下拉框递归生成
     * @todo 无
     * @author 小黄牛
     * @version v1.2.16 + 2020.10.27
     * @deprecated 暂不启用
     * @global 无
     * @param array $list 需要递归的下拉数组
     * @param int $level 递归深度
     * @param string $html 生成的下拉菜单
     * @param string $end_tis 上级父类的树节点符号
     * @return string
    */
    protected function select_menu($list, $level=0, $html='', $end_tis='') {
        foreach ($list as $k=>$v) {
            if ($v['pid'] == 0) {
                $tis = '';
                $level=0;
            } else {
                // 还有同级元素
                if (isset($list[$k+1])) {
                    $tis = '├';
                } else {
                    // 最后一个同级元素
                    $tis = '└';
                }
            }
    
            $span = '';
            for ($i=0; $i<$level; $i++) {
                if ($i > 0) {
                    // 判断是否有下级元素
                    if ($i==($level-1) && isset($v['nodes'])) {
                        // 只有下级还有下级，最后一个竖线才显示
                        if (isset($v['nodes'][0]['nodes']) || $tis == '├') {
                            $span .= '│'; 
                        } else {
                            if ($tis == '└' && $end_tis == '├') {
                                $span .= '│';  
                            } else {
                                $span .= '&nbsp;&nbsp;'; 
                            }
                        }
                    } else {
                        $span .= '│';
                    }
                }
                $span .= '&nbsp;&nbsp;';
            } 
            $html .= '<option value="'.$v['id'].'">'.$span.$tis.$v['text'].'</option>';
            if (isset($v['nodes'])) {
                $html = $this->select_menu($v['nodes'], $level+1, $html, $tis);
            }
        }
        return $html;
    }

    /**
     * 递归查询所有部门的子节点
     * @todo 无
     * @author 小黄牛
     * @version v1.2.16 + 2020.10.27
     * @deprecated 暂不启用
     * @global 无
     * @param string $id
     * @return void
    */
    protected function for_branch($id, $in='') {
        if (!$in) $in = $id;
        $list = \think\Db::name('branch')->where('pid', $id)->field('id')->select();
        foreach ($list as $v) {
            $in .= ','.$v['id'];
            $in = $this->for_branch($v['id'], $in);
        }
        return $in;
    }
    
    /**
     * 递归查询所有岗位的子节点
     * @todo 无
     * @author 小黄牛
     * @version v1.2.16 + 2020.10.27
     * @deprecated 暂不启用
     * @global 无
     * @param string $id
     * @return void
    */
    protected function for_role($id, $in='') {
        if (!$in) $in = $id;
        $list = \think\Db::name('role')->where('pid', $id)->field('id')->select();
        foreach ($list as $v) {
            $in .= ','.$v['id'];
            $in = $this->for_role($v['id'], $in);
        }
        return $in;
    }

    /**
     * 根据权限获取当前账号的部门归属
     * @todo 无
     * @author 小黄牛
     * @version v1.2.16 + 2020.10.27
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    protected function get_branch($role_id=null) {
        $id = 0;
        if ($role_id) {
            $id = $this->for_role($role_id);
        }
        $list = \think\Db::name('role')->order('id ASC, pid ASC')->field('id,pid,role_title as text')->select();//whereIn('id', $id)->
        foreach ($list as $k=>$v) {
            if ($v['pid'] == 0) {
                break;
            } else {
                if ($k==count($list)-1) {
                    $list[0]['pid'] = 0;
                }
            }
        }
        return $list;
    }

    /**
     * 根据权限获取当前账号的岗位归属
     * @todo 无
     * @author 小黄牛
     * @version v1.2.16 + 2020.10.27
     * @deprecated 暂不启用
     * @global 无
     * @return void
    */
    protected function get_role() {
        $id = $this->for_branch($this->admin['branch_id']);
        $list = \think\Db::name('branch')->order('id ASC, pid ASC')->field('id,pid,branch_title as text')->whereIn('id', $id)->select();
        foreach ($list as $k=>$v) {
            if ($v['pid'] == 0) {
                break;
            } else {
                if ($k==count($list)-1) {
                    $list[0]['pid'] = 0;
                }
            }
        }
        return $list;
    }
}