<?php
// +----------------------------------------------------------------------
// | ThinkPHP [ WE CAN DO IT JUST THINK ]
// +----------------------------------------------------------------------
// | Copyright (c) 2006~2018 http://thinkphp.cn All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: liu21st <liu21st@gmail.com>
// +----------------------------------------------------------------------

// +----------------------------------------------------------------------
// | 架构设置
// +----------------------------------------------------------------------

return [
    // 日志调试模式
    'log_debug'      => true,
    
    // +----------------------------------------------------------------------
    // | WebSocket 常用配置
    // +----------------------------------------------------------------------

    // WebSocket 地址，0.0.0.0表示允许接收所有请求
    'websocket_host' => '0.0.0.0',
    // WebSocket 端口
    'websocket_port' => '9502',
    // WebSocket 最大工作进程数
    'websocket_max_process' => 4,
    // WebSocket 最大服务端心跳重连次数
    'websocket_max_heartbeat'  => 3,
    // WebSocket 心跳重连间隔时间(S)
    'websocket_outime_heartbeat' => 10,
    // WebSocket 最大用户在线数
    'websocket_max_online' => 5000,
    // WebSocket AES 加密类型
    'websocket_aes_type' => 'AES-128-ECB',
    // WebSocket AES 加密KEY
    'websocket_aes_key' => '',
    // WebSocket AES 加密向量
    'websocket_aes_iv' => '',
    // WebScoket 外部接口调用时的密钥
    'websocket_api_key' => '123456',
    // WebSocket 外部接口跨域设置
    'websocket_api_origin' => '*',
    // WebSocket 外部接口数据请求类型
    'websocket_api_type' => '',
    // WebSocket 外部接口跨域允许请求的类型
    'websocket_api_methods' => 'POST,GET,OPTIONS,DELETE',
    // WebSocket 外部接口是否允许发送 cookies
    'websocket_api_credentials' => 'true',
    // WebSocket 外部接口允许自定义请求头的字段
    'websocket_api_headers' => 'Content-Type,Content-Length,Accept-Encoding,X-Requested-with, Origin, api_key',
    // WebSocket 进程名称
    'websocket_api_process_name' => 'swoole_server_master',
    // WebSocket SSL证书 CRT
    'websocket_crt_path' => '',
    // WebSocket SSL证书 KEY
    'websocket_key_path' => '',

    // +----------------------------------------------------------------------
    // | WebSocket Redis连接配置
    // +----------------------------------------------------------------------

    // WebScoket Redis host
    'websocket_redis_host' => '127.0.0.1',
    // WebScoket Redis 端口 
    'websocket_redis_port' => '123456',
    // WebScoket Redis 密码 
    'websocket_redis_pwd' => 'redis',
    // WebScoket Redis 库前缀 
    'websocket_redis_table' => 'driver_',
    // WebScoket Redis 连接的超时时间，默认为全局的协程 1S 
    'websocket_redis_connect_timeout' => 1,
    // WebScoket Redis 超时时间，默认为全局的协程 (-1, 永不超时)
    'websocket_redis_timeout' => -1,
    // WebScoket Redis 自动序列化，默认关闭
    'websocket_redis_serialize' => false,
    // WebScoket Redis 自动连接尝试次数 默认1次
    'websocket_redis_reconnect' => 1,
    // WebScoket Redis 是否兼容php/redis原生写法 默认关闭
    'websocket_redis_compatibility_mode' => true,

    // +----------------------------------------------------------------------
    // | WebSocket Redis消息中心配置
    // +----------------------------------------------------------------------
    
    // WebScoket Redis 消息中心前缀 
    'websocket_redis_message_center' => 'driver_message_center_',
    // WebScoket Redis 消息中心失败队列表名
    'websocket_redis_message_center_table' => 'driver_message_center',
    // WebScoket Redis 消息中心失败队列可重复最大次数
    'websocket_redis_message_center_num' => 3,
    // WebScoket Redis 消息中心失败队列重复时间 (S)
    'websocket_redis_message_center_work_time' => 10,
    // WebScoket Redis 消息中心失败队列 每次重复条数
    'websocket_redis_message_center_work_max' => 300,

    // +----------------------------------------------------------------------
    // | WebSocket Redis连接池配置
    // +----------------------------------------------------------------------
    
    // 空闲回收定时检查时间 (S)
    'redis_timing_recovery' => 600,
    // WebScoket Redis 最小连接数
    'websocket_redis_pool_min' => 10,
    // WebScoket Redis 最大连接数
    'websocket_redis_pool_max' => 100,
    // WebScoket Redis 空闲连接回收时间
    'websocket_redis_pool_spare_time' => 3600,

    
    // +----------------------------------------------------------------------
    // | 数据库连接池配置 - 读写分离
    // +----------------------------------------------------------------------    
    // 空闲回收定时检查时间 (S)
    'mysql_timing_recovery' => 600,

    # 读
    // 最小连接数
    'pool_read_min' => 5,
    // 最大连接数
    'pool_read_max' => 10,
    // 空闲连接回收时间
    'pool_read_spare_time' => 3600,
    
    # 写
    // 最小连接数
    'pool_write_min' => 5,
    // 最大连接数
    'pool_write_max' => 10,
    // 空闲连接回收时间
    'pool_write_spare_time' => 3600,

    # 日志
    // 最小连接数
    'pool_log_min' => 5,
    // 最大连接数
    'pool_log_max' => 10,
    // 空闲连接回收时间
    'pool_log_spare_time' => 3600,

    // +----------------------------------------------------------------------
    // | 数据库配置 - 读 - 注意：会循环使用配置创建连接池
    // +----------------------------------------------------------------------
    'pool_read_database' => [
        [
            'host'     => '127.0.0.1', // 地址
            'port'     => '3306', // 端口
            'user'     => 'root', // 用户名
            'password' => 'root', // 密码
            'database' => 'websocket', // 库
            'charset'  => 'utf8mb4', // 字符集
            'timeout'  => 10, // 连接超时时间
        ],
    ],
    // +----------------------------------------------------------------------
    // | 数据库配置 - 写 - 注意：会循环使用配置创建连接池
    // +----------------------------------------------------------------------
    'pool_write_database' => [
        [
            'host'     => '127.0.0.1', // 地址
            'port'     => '3306', // 端口
            'user'     => 'root', // 用户名
            'password' => 'root', // 密码
            'database' => 'websocket', // 库
            'charset'  => 'utf8mb4', // 字符集
            'timeout'  => 10, // 连接超时时间
        ],
    ],
    // +----------------------------------------------------------------------
    // | 数据库配置 - 日志 - 注意：会循环使用配置创建连接池
    // +----------------------------------------------------------------------
    'pool_log_database' => [
        [
            'host'     => '127.0.0.1', // 地址
            'port'     => '3306', // 端口
            'user'     => 'root', // 用户名
            'password' => 'root', // 密码
            'database' => 'websocket', // 库
            'charset'  => 'utf8mb4', // 字符集
            'timeout'  => 10, // 连接超时时间
        ],
    ],
];
